@@ -1,6 +1,5 @@
 #This file is part of Tryton.  The COPYRIGHT file at the top level of
 #this repository contains the full copyright notices and license terms.
-
 from trytond.backend.database import DatabaseInterface, CursorInterface
 from trytond.config import CONFIG
 from trytond.session import Session
@@ -9,6 +8,8 @@ import re
 from decimal import Decimal
 import datetime
 import time
+import sys
+import threading
 
 _FIX_ROWCOUNT = False
 try:
@@ -88,19 +89,20 @@ def replace(text, pattern, replacement):
 
 class Database(DatabaseInterface):
 
-    _memory_database = None
+    _local = threading.local()
     _conn = None
 
     def __new__(cls, database_name=':memory:'):
-        if database_name == ':memory:' \
-                and cls._memory_database:
-            return cls._memory_database
+        if (database_name == ':memory:'
+                and hasattr(cls._local, 'memory_database')
+                and cls._local.memory_database):
+            return cls._local.memory_database
         return DatabaseInterface.__new__(cls, database_name=database_name)
 
     def __init__(self, database_name=':memory:'):
         super(Database, self).__init__(database_name=database_name)
         if database_name == ':memory:':
-            Database._memory_database = self
+            Database._local.memory_database = self
 
     def connect(self):
         if self.database_name == ':memory:':
@@ -120,7 +122,7 @@ class Database(DatabaseInterface):
             self._conn.create_function('replace', 3, replace)
         return self
 
-    def cursor(self, autocommit=False):
+    def cursor(self, autocommit=False, readonly=False):
         if self._conn is None:
             self.connect()
         if autocommit:
@@ -144,10 +146,9 @@ class Database(DatabaseInterface):
                 return
             path = os.path.join(CONFIG['data_path'],
                     database_name + '.sqlite')
-        conn = sqlite.connect(path)
-        cursor = conn.cursor()
-        cursor.close()
-        conn.close()
+        with sqlite.connect(path) as conn:
+            cursor = conn.cursor()
+            cursor.close()
 
     def drop(self, cursor, database_name):
         if database_name == ':memory:':
@@ -166,9 +167,8 @@ class Database(DatabaseInterface):
             raise Exception('Wrong database name!')
         path = os.path.join(CONFIG['data_path'],
                 database_name + '.sqlite')
-        file_p = open(path, 'rb')
-        data = file_p.read()
-        file_p.close()
+        with open(path, 'rb') as file_p:
+            data = file_p.read()
         return data
 
     @staticmethod
@@ -181,9 +181,8 @@ class Database(DatabaseInterface):
                 database_name + '.sqlite')
         if os.path.isfile(path):
             raise Exception('Database already exists!')
-        file_p = open(path, 'wb')
-        file_p.write(data)
-        file_p.close()
+        with open(path, 'wb') as file_p:
+            file_p.write(data)
 
     @staticmethod
     def list(cursor):
@@ -213,15 +212,17 @@ class Database(DatabaseInterface):
     def init(cursor):
         from trytond.tools import safe_eval
         sql_file = os.path.join(os.path.dirname(__file__), 'init.sql')
-        for line in open(sql_file).read().split(';'):
-            if (len(line)>0) and (not line.isspace()):
-                cursor.execute(line)
+        with open(sql_file) as fp:
+            for line in fp.read().split(';'):
+                if (len(line)>0) and (not line.isspace()):
+                    cursor.execute(line)
 
         for i in ('ir', 'workflow', 'res', 'webdav'):
             root_path = os.path.join(os.path.dirname(__file__), '..', '..')
             tryton_file = os.path.join(root_path, i, '__tryton__.py')
             mod_path = os.path.join(root_path, i)
-            info = safe_eval(open(tryton_file).read())
+            with open(tryton_file) as fp:
+                info = safe_eval(fp.read())
             active = info.get('active', False)
             if active:
                 state = 'to install'
@@ -292,8 +293,7 @@ class Cursor(CursorInterface):
         sql = buf[:-2]
         try:
             if params:
-                res = self.cursor.execute(sql, [isinstance(x, str) and \
-                        unicode(x, 'utf-8') or x for x in  params])
+                res = self.cursor.execute(sql, params)
             else:
                 res = self.cursor.execute(sql)
         except Exception:
@@ -358,7 +358,10 @@ class Cursor(CursorInterface):
         return select
 
 sqlite.register_converter('NUMERIC', lambda val: Decimal(val))
-sqlite.register_adapter(Decimal, lambda val: buffer(str(val)))
+if sys.version_info[0] == 2:
+    sqlite.register_adapter(Decimal, lambda val: buffer(str(val)))
+else:
+    sqlite.register_adapter(Decimal, lambda val: bytes(str(val)))
 sqlite.register_adapter(Session, int)
 def adapt_datetime(val):
     return val.replace(tzinfo=None).isoformat(" ")
