@@ -1,22 +1,22 @@
 #This file is part of Tryton.  The COPYRIGHT file at the top level of
 #this repository contains the full copyright notices and license terms.
-from __future__ import with_statement
 import contextlib
-import base64
 try:
     import cStringIO as StringIO
 except ImportError:
     import StringIO
 import zipfile
+import polib
+import operator
 from xml import dom
 from xml.dom import minidom
 from difflib import SequenceMatcher
-import csv
 import os
 try:
     from hashlib import md5
 except ImportError:
     from md5 import md5
+from functools import reduce
 from trytond.model import ModelView, ModelSQL, fields
 from trytond.model.cacheable import Cacheable
 from trytond.wizard import Wizard
@@ -25,6 +25,7 @@ from trytond.tools import file_open, reduce_ids
 from trytond.backend import TableHandler, FIELDS
 from trytond.pyson import PYSONEncoder
 from trytond.transaction import Transaction
+from trytond.pool import Pool
 
 TRANSLATION_TYPE = [
     ('field', 'Field'),
@@ -37,13 +38,12 @@ TRANSLATION_TYPE = [
     ('error', 'Error'),
 ]
 
-HEADER = ['type', 'name', 'res_id', 'src', 'value', 'fuzzy']
 
+class TrytonPOFile(polib.POFile):
 
-class TRYTON(csv.excel):
-    lineterminator = '\n'
-
-csv.register_dialect("TRYTON", TRYTON)
+    def sort(self):
+        return super(TrytonPOFile, self).sort(
+            key=lambda x: (x.msgctxt, x.msgid))
 
 
 class Translation(ModelSQL, ModelView, Cacheable):
@@ -70,12 +70,6 @@ class Translation(ModelSQL, ModelView, Cacheable):
                 'UNIQUE (name, res_id, lang, type, src_md5, module)',
                 'Translation must be unique'),
         ]
-        self._constraints += [
-            ('check_unique_model', 'unique_model'),
-        ]
-        self._error_messages.update({
-            'unique_model': "Translation of type 'model' must be unique!",
-            })
         self._max_len = 10240
 
     def init(self, module_name):
@@ -115,20 +109,6 @@ class Translation(ModelSQL, ModelView, Cacheable):
     def default_fuzzy(self):
         return False
 
-    def check_unique_model(self, ids):
-        "Check unique model"
-        cursor = Transaction().cursor
-        query = ('SELECT count(1) FROM "%s" '
-                'WHERE type = %%s '
-                    'AND res_id != 0 '
-                'GROUP BY name, res_id, lang, type, src '
-                'HAVING count(1) > 1' % self._table)
-        cursor.execute(query, ('model',))
-        rowcount = cursor.rowcount
-        if rowcount == -1 or rowcount is None:
-            rowcount = len(cursor.fetchall())
-        return not bool(rowcount)
-
     def get_model(self, ids, name):
         res = {}
         for translation in self.browse(ids):
@@ -143,7 +123,8 @@ class Translation(ModelSQL, ModelView, Cacheable):
         return [('id', 'in', [x[0] for x in cursor.fetchall()])]
 
     def get_language(self):
-        lang_obj = self.pool.get('ir.lang')
+        pool = Pool()
+        lang_obj = pool.get('ir.lang')
         lang_ids = lang_obj.search([])
         langs = lang_obj.browse(lang_ids)
         res = [(lang.code, lang.name) for lang in langs]
@@ -153,8 +134,9 @@ class Translation(ModelSQL, ModelView, Cacheable):
         return md5((src or '').encode('utf-8')).hexdigest()
 
     def _get_ids(self, name, ttype, lang, ids):
-        model_fields_obj = self.pool.get('ir.model.field')
-        model_obj = self.pool.get('ir.model')
+        pool = Pool()
+        model_fields_obj = pool.get('ir.model.field')
+        model_obj = pool.get('ir.model')
 
         translations, to_fetch = {}, []
         name = unicode(name)
@@ -220,8 +202,9 @@ class Translation(ModelSQL, ModelView, Cacheable):
         return translations
 
     def _set_ids(self, name, ttype, lang, ids, value):
-        model_fields_obj = self.pool.get('ir.model.field')
-        model_obj = self.pool.get('ir.model')
+        pool = Pool()
+        model_fields_obj = pool.get('ir.model.field')
+        model_obj = pool.get('ir.model')
 
         if lang == 'en_US':
             return 0
@@ -262,7 +245,7 @@ class Translation(ModelSQL, ModelView, Cacheable):
                             'fuzzy': False,
                             })
             return len(ids)
-        model_obj = self.pool.get(model_name)
+        model_obj = pool.get(model_name)
         for record in model_obj.browse(ids):
             ids2 = self.search([
                 ('lang', '=', lang),
@@ -438,8 +421,9 @@ class Translation(ModelSQL, ModelView, Cacheable):
             vals['src_md5'] = self.get_src_md5(vals.get('src'))
         return super(Translation, self).write(ids, vals)
 
-    def translation_import(self, lang, module, datas):
-        model_data_obj = self.pool.get('ir.model.data')
+    def translation_import(self, lang, module, po_path):
+        pool = Pool()
+        model_data_obj = pool.get('ir.model.data')
         model_data_ids = model_data_obj.search([
             ('module', '=', module),
             ])
@@ -449,9 +433,7 @@ class Translation(ModelSQL, ModelView, Cacheable):
             fs_id2model_data[model_data.model][model_data.fs_id] = model_data
 
         translation_ids = []
-        reader = csv.reader(datas)
-        for row in reader:
-            break
+        pofile = polib.pofile(po_path)
 
         id2translation = {}
         key2ids = {}
@@ -471,13 +453,11 @@ class Translation(ModelSQL, ModelView, Cacheable):
             key2ids.setdefault(key, []).append(translation.id)
             id2translation[translation.id] = translation
 
-        for row in reader:
-            ttype = row[0].decode('utf-8')
-            name = row[1].decode('utf-8')
-            res_id = row[2].decode('utf-8')
-            src = row[3].decode('utf-8')
-            value = row[4].decode('utf-8')
-            fuzzy = bool(int(row[5]))
+        for entry in pofile:
+            ttype, name, res_id = entry.msgctxt.split(':')
+            src = entry.msgid
+            value = entry.msgstr
+            fuzzy = 'fuzzy' in entry.flags
             noupdate = False
 
             model = name.split(',')[0]
@@ -537,7 +517,8 @@ class Translation(ModelSQL, ModelView, Cacheable):
         return len(translation_ids)
 
     def translation_export(self, lang, module):
-        model_data_obj = self.pool.get('ir.model.data')
+        pool = Pool()
+        model_data_obj = pool.get('ir.model.data')
 
         model_data_ids = model_data_obj.search([
             ('module', '=', module),
@@ -547,10 +528,10 @@ class Translation(ModelSQL, ModelView, Cacheable):
             db_id2fs_id.setdefault(model_data.model, {})
             db_id2fs_id[model_data.model][model_data.db_id] = model_data.fs_id
 
-        buf = StringIO.StringIO()
-        writer = csv.writer(buf, 'TRYTON')
-        writer.writerow(HEADER)
-        rows = []
+        pofile = TrytonPOFile(wrapwidth=78)
+        pofile.metadata = {
+            'Content-Type': 'text/plain; charset=utf-8',
+            }
 
         with Transaction().set_context(language='en_US'):
             translation_ids = self.search([
@@ -558,32 +539,22 @@ class Translation(ModelSQL, ModelView, Cacheable):
                 ('module', '=', module),
                 ], order=[])
         for translation in self.browse(translation_ids):
-            row = []
-            for field in HEADER:
-                if field == 'res_id':
-                    res_id = translation[field]
-                    if res_id:
-                        model = translation.name.split(',')[0]
-                        if model in db_id2fs_id:
-                            res_id = db_id2fs_id[model].get(res_id)
-                        else:
-                            break
-                    row.append(res_id)
-                elif field == 'fuzzy':
-                    row.append(int(translation[field]))
+            flags = [] if not translation['fuzzy'] else ['fuzzy']
+            trans_ctxt = '%(type)s:%(name)s' % translation
+            res_id = translation['res_id']
+            if res_id:
+                model, _ = translation.name.split(',')
+                if model in db_id2fs_id:
+                    res_id = db_id2fs_id[model].get(res_id)
                 else:
-                    value = translation[field] or ''
-                    value = value.encode('utf-8')
-                    row.append(value)
-            if len(row) == len(HEADER):
-                rows.append(row)
+                    continue
+            trans_ctxt += ':%s' % res_id
+            entry = polib.POEntry(msgid=(translation.src or ''),
+                msgstr=(translation.value or ''), msgctxt=trans_ctxt, flags=flags)
+            pofile.append(entry)
 
-        rows.sort()
-        writer.writerows(rows)
-
-        file_data = buf.getvalue()
-        buf.close()
-        return file_data
+        pofile.sort()
+        return unicode(pofile).encode('utf-8')
 
 Translation()
 
@@ -651,8 +622,9 @@ class ReportTranslationSet(Wizard):
         return strings
 
     def _set_report_translation(self, data):
-        report_obj = self.pool.get('ir.action.report')
-        translation_obj = self.pool.get('ir.translation')
+        pool = Pool()
+        report_obj = pool.get('ir.action.report')
+        translation_obj = pool.get('ir.translation')
 
         with Transaction().set_context(active_test=False):
             report_ids = report_obj.search([])
@@ -680,14 +652,14 @@ class ReportTranslationSet(Wizard):
                 with file_open(report.report.replace('/', os.sep),
                         mode='rb') as fp:
                     odt_content = fp.read()
-            for content in (report.report_content_data, odt_content):
+            for content in (report.report_content_custom, odt_content):
                 if not content:
                     continue
 
                 content_io = StringIO.StringIO(content)
                 try:
                     content_z = zipfile.ZipFile(content_io, mode='r')
-                except BadZipfile:
+                except zipfile.BadZipfile:
                     continue
 
                 content_xml = content_z.read('content.xml')
@@ -698,14 +670,8 @@ class ReportTranslationSet(Wizard):
                 document = dom.minidom.parseString(style_xml)
                 strings += self._translate_report(document.documentElement)
 
-            style_content = None
-            try:
-                style_content = base64.decodestring(report.style_content)
-            except Exception:
-                pass
-
-            if style_content:
-                style_io = StringIO.StringIO(style_content)
+            if report.style_content:
+                style_io = StringIO.StringIO(report.style_content)
                 style_z = zipfile.ZipFile(style_io, mode='r')
                 style_xml = style_z.read('styles.xml')
 
@@ -807,189 +773,178 @@ class TranslationClean(Wizard):
         },
     }
 
+    def _clean_field(self, translation):
+        pool = Pool()
+        try:
+            model_name, field_name = translation.name.split(',', 1)
+        except ValueError:
+            return True
+        if model_name not in pool.object_name_list():
+            return True
+        model_obj = pool.get(model_name)
+        if field_name not in model_obj._columns:
+            return True
+
+    def _clean_model(self, translation):
+        pool = Pool()
+        try:
+            model_name, field_name = translation.name.split(',', 1)
+        except ValueError:
+            return True
+        if model_name not in pool.object_name_list():
+            return True
+        if translation.res_id:
+            model_obj = pool.get(model_name)
+            if field_name not in model_obj._columns:
+                return True
+            field = model_obj._columns[field_name]
+            if (not hasattr(field, 'translate')
+                    or not field.translate):
+                return True
+        elif field_name not in ('name'):
+            return True
+
+    def _clean_odt(self, translation):
+        pool = Pool()
+        report_obj = pool.get('ir.action.report')
+        with Transaction().set_context(active_test=False):
+            if not report_obj.search([
+                        ('report_name', '=', translation.name),
+                        ]):
+                return True
+
+    def _clean_selection(self, translation):
+        pool = Pool()
+        try:
+            model_name, field_name = translation.name.split(',', 1)
+        except ValueError:
+            return True
+        if model_name not in pool.object_name_list():
+            return True
+        model_obj = pool.get(model_name)
+        if field_name not in model_obj._columns:
+            return True
+        field = model_obj._columns[field_name]
+        if (not hasattr(field, 'selection')
+                or not field.selection
+                or not getattr(field, 'translate_selection', True)):
+            return True
+        if (isinstance(field.selection, (tuple, list))
+                and translation.src not in dict(field.selection).values()):
+            return True
+
+    def _clean_view(self, translation):
+        pool = Pool()
+        model_name = translation.name
+        if model_name not in pool.object_name_list():
+            return True
+
+    def _clean_wizard_button(self, translation):
+        pool = Pool()
+        try:
+            wizard_name, state_name, button_name = \
+                    translation.name.split(',', 2)
+        except ValueError:
+            return True
+        if (wizard_name not in
+                pool.object_name_list(type='wizard')):
+            return True
+        wizard = pool.get(wizard_name, type='wizard')
+        if not wizard:
+            return True
+        state = wizard.states.get(state_name)
+        if not state:
+            return True
+        for but in state['result']['state']:
+            if but[0] == button_name:
+                return False
+        return True
+
+    def _clean_help(self, translation):
+        pool = Pool()
+        try:
+            model_name, field_name = translation.name.split(',', 1)
+        except ValueError:
+            return True
+        if model_name not in pool.object_name_list():
+            return True
+        model_obj = pool.get(model_name)
+        if field_name not in model_obj._columns:
+            return True
+        field = model_obj._columns[field_name]
+        return not field.help
+
+    def _clean_error(self, translation):
+        pool = Pool()
+        model_name = translation.name
+        if model_name in (
+                'delete_xml_record',
+                'xml_record_desc',
+                'write_xml_record',
+                'not_found_in_selection',
+                'relation_not_found',
+                'too_many_relations_found',
+                'xml_id_syntax_error',
+                'reference_syntax_error',
+                'delete_workflow_record',
+                'domain_validation_record',
+                'required_validation_record',
+                'size_validation_record',
+                'digits_validation_record',
+                'access_error',
+                'read_error',
+                'write_error',
+                'required_field',
+                'foreign_model_missing',
+                'foreign_model_exist',
+                'search_function_missing',
+                ):
+            return False
+        if model_name in pool.object_name_list():
+            model_obj = pool.get(model_name)
+            errors = model_obj._error_messages.values() + \
+                    model_obj._sql_error_messages.values()
+            for _, _, error in model_obj._sql_constraints:
+                errors.append(error)
+            if translation.src not in errors:
+                return True
+        elif model_name in pool.object_name_list(type='wizard'):
+            wizard_obj = pool.get(model_name, type='wizard')
+            errors = wizard_obj._error_messages.values()
+            if translation.src not in errors:
+                return True
+        else:
+            return True
+
     def _clean_translation(self, data):
-        translation_obj = self.pool.get('ir.translation')
-        model_data_obj = self.pool.get('ir.model.data')
-        report_obj = self.pool.get('ir.action.report')
+        pool = Pool()
+        translation_obj = pool.get('ir.translation')
+        model_data_obj = pool.get('ir.model.data')
 
-        offset = 0
-        cursor = Transaction().cursor
-        limit = cursor.IN_MAX
-        while True:
-            to_delete = []
-            translation_ids = translation_obj.search([], offset=offset,
-                    limit=limit)
-            if not translation_ids:
-                break
-            offset += limit
-            for translation in translation_obj.browse(translation_ids):
-                if translation.type == 'field':
-                    try:
-                        model_name, field_name = translation.name.split(',', 1)
-                    except ValueError:
-                        to_delete.append(translation.id)
-                        continue
-                    if model_name not in self.pool.object_name_list():
-                        to_delete.append(translation.id)
-                        continue
-                    model_obj = self.pool.get(model_name)
-                    if field_name not in model_obj._columns:
-                        to_delete.append(translation.id)
-                        continue
-                elif translation.type == 'model':
-                    try:
-                        model_name, field_name = translation.name.split(',', 1)
-                    except ValueError:
-                        to_delete.append(translation.id)
-                        continue
-                    if model_name not in self.pool.object_name_list():
-                        to_delete.append(translation.id)
-                        continue
-                    if translation.res_id:
-                        model_obj = self.pool.get(model_name)
-                        if field_name not in model_obj._columns:
-                            to_delete.append(translation.id)
-                            continue
-                        field = model_obj._columns[field_name]
-                        if not hasattr(field, 'translate') or \
-                                not field.translate:
-                            to_delete.append(translation.id)
-                            continue
-                    elif field_name not in ('name'):
-                        to_delete.append(translation.id)
-                        continue
-                elif translation.type == 'odt':
-                    with Transaction().set_context(active_test=False):
-                        if not report_obj.search([
-                            ('report_name', '=', translation.name),
-                            ]):
-                            to_delete.append(translation.id)
-                            continue
-                elif translation.type == 'selection':
-                    try:
-                        model_name, field_name = translation.name.split(',', 1)
-                    except ValueError:
-                        to_delete.append(translation.id)
-                        continue
-                    if model_name not in self.pool.object_name_list():
-                        to_delete.append(translation.id)
-                        continue
-                    model_obj = self.pool.get(model_name)
-                    if field_name not in model_obj._columns:
-                        to_delete.append(translation.id)
-                        continue
-                    field = model_obj._columns[field_name]
-                    if not hasattr(field, 'selection') or not field.selection \
-                            or not ((hasattr(field, 'translate_selection') and \
-                            field.translate_selection) or True):
-                        to_delete.append(translation.id)
-                        continue
-                    if isinstance(field.selection, (tuple, list)) \
-                            and translation.src not in \
-                            dict(field.selection).values():
-                        to_delete.append(translation.id)
-                        continue
-                elif translation.type == 'view':
-                    model_name = translation.name
-                    if model_name not in self.pool.object_name_list():
-                        to_delete.append(translation.id)
-                        continue
-                elif translation.type == 'wizard_button':
-                    try:
-                        wizard_name, state_name, button_name = \
-                                translation.name.split(',', 2)
-                    except ValueError:
-                        to_delete.append(translation.id)
-                        continue
-                    if wizard_name not in \
-                            self.pool.object_name_list(type='wizard'):
-                        to_delete.append(translation.id)
-                        continue
-                    wizard = self.pool.get(wizard_name, type='wizard')
-                    if not wizard:
-                        to_delete.append(translation.id)
-                        continue
-                    state = wizard.states.get(state_name)
-                    if not state:
-                        to_delete.append(translation.id)
-                        continue
-                    find = False
-                    for but in state['result']['state']:
-                        if but[0] == button_name:
-                            find = True
-                    if not find:
-                        to_delete.append(translation.id)
-                        continue
-                elif translation.type == 'help':
-                    try:
-                        model_name, field_name = translation.name.split(',', 1)
-                    except ValueError:
-                        to_delete.append(translation.id)
-                        continue
-                    if model_name not in self.pool.object_name_list():
-                        to_delete.append(translation.id)
-                        continue
-                    model_obj = self.pool.get(model_name)
-                    if field_name not in model_obj._columns:
-                        to_delete.append(translation.id)
-                        continue
-                    field = model_obj._columns[field_name]
-                    if not field.help:
-                        to_delete.append(translation.id)
-                        continue
-                elif translation.type == 'error':
-                    model_name = translation.name
-                    if model_name in (
-                            'delete_xml_record',
-                            'xml_record_desc',
-                            'write_xml_record',
-                            'not_found_in_selection',
-                            'relation_not_found',
-                            'too_many_relations_found',
-                            'xml_id_syntax_error',
-                            'reference_syntax_error',
-                            'delete_workflow_record',
-                            'domain_validation_record',
-                            'required_validation_record',
-                            'size_validation_record',
-                            'digits_validation_record',
-                            'access_error',
-                            'read_error',
-                            'write_error',
-                            'required_field',
-                            'foreign_model_missing',
-                            'foreign_model_exist',
-                            'search_function_missing',
-                            ):
-                        continue
-                    if model_name in self.pool.object_name_list():
-                        model_obj = self.pool.get(model_name)
-                        errors = model_obj._error_messages.values() + \
-                                model_obj._sql_error_messages.values()
-                        for _, _, error in model_obj._sql_constraints:
-                            errors.append(error)
-                        if translation.src not in errors:
-                            to_delete.append(translation.id)
-                            continue
-                    elif model_name in self.pool.object_name_list(type='wizard'):
-                        wizard_obj = self.pool.get(model_name, type='wizard')
-                        errors = wizard_obj._error_messages.values()
-                        if translation.src not in errors:
-                            to_delete.append(translation.id)
-                            continue
-                    else:
-                        to_delete.append(translation.id)
-                        continue
-            # skip translation handled in ir.model.data
-            mdata_ids = model_data_obj.search([
-                ('db_id', 'in', to_delete),
-                ('model', '=', 'ir.translation'),
-                ])
-            for mdata in model_data_obj.browse(mdata_ids):
-                if mdata.db_id in to_delete:
-                    to_delete.remove(mdata.db_id)
+        to_delete = []
+        keys = set()
+        translation_ids = translation_obj.search([])
+        for translation in translation_obj.browse(translation_ids):
+            if getattr(self, '_clean_%s' % translation.type)(translation):
+                to_delete.append(translation.id)
+            elif translation.type in ('field', 'model', 'wizard_button',
+                'help'):
+                key = (translation.module, translation.lang, translation.type,
+                    translation.name, translation.res_id)
+                if key in keys:
+                    to_delete.append(translation.id)
+                else:
+                    keys.add(key)
+        # skip translation handled in ir.model.data
+        mdata_ids = model_data_obj.search([
+            ('db_id', 'in', to_delete),
+            ('model', '=', 'ir.translation'),
+            ])
+        for mdata in model_data_obj.browse(mdata_ids):
+            if mdata.db_id in to_delete:
+                to_delete.remove(mdata.db_id)
 
-            translation_obj.delete(to_delete)
+        translation_obj.delete(to_delete)
         return {}
 
 TranslationClean()
@@ -1006,7 +961,8 @@ class TranslationUpdateInit(ModelView):
         return Transaction().context.get('language', False)
 
     def get_language(self):
-        lang_obj = self.pool.get('ir.lang')
+        pool = Pool()
+        lang_obj = pool.get('ir.lang')
         lang_ids = lang_obj.search([('translatable', '=', True)])
         langs = lang_obj.browse(lang_ids)
         res = [(lang.code, lang.name) for lang in langs if lang.code != 'en_US']
@@ -1020,7 +976,8 @@ class TranslationUpdate(Wizard):
     _name = "ir.translation.update"
 
     def _update_translation(self, data):
-        translation_obj = self.pool.get('ir.translation')
+        pool = Pool()
+        translation_obj = pool.get('ir.translation')
         cursor = Transaction().cursor
         cursor.execute('SELECT name, res_id, type, src, module ' \
                 'FROM ir_translation ' \
@@ -1108,8 +1065,9 @@ class TranslationUpdate(Wizard):
         return {}
 
     def _action_translation_open(self, data):
-        model_data_obj = self.pool.get('ir.model.data')
-        act_window_obj = self.pool.get('ir.action.act_window')
+        pool = Pool()
+        model_data_obj = pool.get('ir.model.data')
+        act_window_obj = pool.get('ir.action.act_window')
 
         model_data_ids = model_data_obj.search([
             ('fs_id', '=', 'act_translation_form'),
@@ -1162,7 +1120,8 @@ class TranslationExportInit(ModelView):
         return Transaction().context.get('language', False)
 
     def get_language(self):
-        lang_obj = self.pool.get('ir.lang')
+        pool = Pool()
+        lang_obj = pool.get('ir.lang')
         lang_ids = lang_obj.search([
             ('translatable', '=', True),
             ])
@@ -1171,7 +1130,8 @@ class TranslationExportInit(ModelView):
         return res
 
     def get_module(self):
-        module_obj = self.pool.get('ir.module.module')
+        pool = Pool()
+        module_obj = pool.get('ir.module.module')
         module_ids = module_obj.search([
             ('state', 'in', ['installed', 'to upgrade', 'to remove']),
             ])
@@ -1196,11 +1156,12 @@ class TranslationExport(Wizard):
     _name = "ir.translation.export"
 
     def _export_translation(self, data):
-        translation_obj = self.pool.get('ir.translation')
+        pool = Pool()
+        translation_obj = pool.get('ir.translation')
         file_data = translation_obj.translation_export(data['form']['lang'],
                 data['form']['module'])
         return {
-            'file': base64.encodestring(file_data),
+            'file': buffer(file_data),
         }
 
     states = {
