@@ -1,8 +1,8 @@
 #This file is part of Tryton.  The COPYRIGHT file at the top level of
 #this repository contains the full copyright notices and license terms.
-from __future__ import with_statement
 from trytond.model import ModelView, ModelSQL, fields
 from trytond.transaction import Transaction
+from trytond.pool import Pool
 
 def one_in(i, j):
     """Check the presence of an element of setA in setB
@@ -12,65 +12,53 @@ def one_in(i, j):
             return True
     return False
 
-ICONS = [(x, x) for x in [
-    'tryton-accessories',
+CLIENT_ICONS = [(x, x) for x in (
+    'tryton-attachment-hi',
     'tryton-attachment',
+    'tryton-cancel',
     'tryton-clear',
     'tryton-close',
-    'tryton-calculator',
-    'tryton-calendar',
-    'tryton-clock',
     'tryton-connect',
     'tryton-copy',
-    'tryton-currency',
     'tryton-delete',
-    'tryton-development',
     'tryton-dialog-error',
     'tryton-dialog-information',
     'tryton-dialog-warning',
     'tryton-disconnect',
     'tryton-executable',
-    'tryton-find',
     'tryton-find-replace',
+    'tryton-find',
     'tryton-folder-new',
-    'tryton-folder-saved-search',
     'tryton-fullscreen',
-    'tryton-graph',
     'tryton-go-home',
     'tryton-go-jump',
     'tryton-go-next',
     'tryton-go-previous',
     'tryton-help',
-    'tryton-image-missing',
-    'tryton-information',
-    'tryton-lock',
-    'tryton-list',
+    'tryton-icon',
     'tryton-list-add',
     'tryton-list-remove',
     'tryton-locale',
+    'tryton-lock',
     'tryton-log-out',
     'tryton-mail-message-new',
+    'tryton-mail-message',
     'tryton-new',
-    'tryton-noimage',
+    'tryton-ok',
     'tryton-open',
-    'tryton-package',
-    'tryton-preferences',
-    'tryton-preferences-system',
     'tryton-preferences-system-session',
-    'tryton-presentation',
+    'tryton-preferences-system',
+    'tryton-preferences',
     'tryton-print',
-    'tryton-readonly',
     'tryton-refresh',
     'tryton-save-as',
     'tryton-save',
-    'tryton-spreadsheet',
     'tryton-start-here',
-    'tryton-tree',
-    'tryton-system',
     'tryton-system-file-manager',
-    'tryton-users',
-    'tryton-web-browser',
-]]
+    'tryton-system',
+    'tryton-undo',
+    'tryton-web-browser')]
+SEPARATOR = ' / '
 
 
 class UIMenu(ModelSQL, ModelView):
@@ -80,12 +68,13 @@ class UIMenu(ModelSQL, ModelView):
     name = fields.Char('Menu', required=True, translate=True)
     sequence = fields.Integer('Sequence')
     childs = fields.One2Many('ir.ui.menu', 'parent', 'Children')
-    parent = fields.Many2One('ir.ui.menu', 'Parent Menu', select=1)
+    parent = fields.Many2One('ir.ui.menu', 'Parent Menu', select=1,
+            ondelete='CASCADE')
     groups = fields.Many2Many('ir.ui.menu-res.group',
-       'menu_id', 'gid', 'Groups')
+       'menu', 'group', 'Groups')
     complete_name = fields.Function(fields.Char('Complete Name',
-        order_field='name'), 'get_full_name')
-    icon = fields.Selection(ICONS, 'Icon', translate=False)
+        order_field='name'), 'get_rec_name', searcher='search_rec_name')
+    icon = fields.Selection('list_icons', 'Icon', translate=False)
     action = fields.Function(fields.Reference('Action',
         selection=[
             ('ir.action.report', 'ir.action.report'),
@@ -98,6 +87,14 @@ class UIMenu(ModelSQL, ModelView):
     def __init__(self):
         super(UIMenu, self).__init__()
         self._order.insert(0, ('sequence', 'ASC'))
+        self._constraints += [
+            ('check_recursion', 'recursive_menu'),
+            ('check_name', 'wrong_name'),
+        ]
+        self._error_messages.update({
+            'recursive_menu': 'You can not create recursive menu!',
+            'wrong_name': 'You can not use "%s" in name field!' % SEPARATOR,
+        })
 
     def default_icon(self):
         return 'tryton-open'
@@ -108,20 +105,46 @@ class UIMenu(ModelSQL, ModelView):
     def default_active(self):
         return True
 
-    def get_full_name(self, ids, name):
-        res = {}
+    def list_icons(self):
+        pool = Pool()
+        icon_obj = pool.get('ir.ui.icon')
+        return sorted(CLIENT_ICONS
+            + [(name, name) for _, name in icon_obj.list_icons()])
+
+    def check_name(self, ids):
         for menu in self.browse(ids):
-            res[menu.id] = self._get_one_full_name(menu)
+            if SEPARATOR in menu.name:
+                return False
+        return True
+
+    def get_rec_name(self, ids, name):
+        if not ids:
+            return {}
+        res = {}
+        def _name(menu):
+            if menu.id in res:
+                return res[menu.id]
+            elif menu.parent:
+                return _name(menu.parent) + SEPARATOR + menu.name
+            else:
+                return menu.name
+        for menu in self.browse(ids):
+            res[menu.id] = _name(menu)
         return res
 
-    def _get_one_full_name(self, menu, level=6):
-        if level <= 0:
-            return '...'
-        if menu.parent:
-            parent_path = self._get_one_full_name(menu.parent, level-1) + "/"
-        else:
-            parent_path = ''
-        return parent_path + menu.name
+    def search_rec_name(self, name, clause):
+        if isinstance(clause[2], basestring):
+            values = clause[2].split(SEPARATOR)
+            values.reverse()
+            domain = []
+            field = 'name'
+            for name in values:
+                domain.append((field, clause[1], name))
+                field = 'parent.' + field
+            ids = self.search(domain, order=[])
+            return [('id', 'in', ids)]
+        #TODO Handle list
+        return [('name',) + tuple(clause[1:])]
 
     def search(self, domain, offset=0, limit=None, order=None, count=False,
             query_string=False):
@@ -130,25 +153,22 @@ class UIMenu(ModelSQL, ModelView):
         if query_string:
             return res
 
-        def check_menu(res):
-            if not res:
-                return []
+        if res:
             menus = self.browse(res)
             parent_ids = [x.parent.id for x in menus if x.parent]
             parent_ids = self.search([
                 ('id', 'in', parent_ids),
                 ])
-            parent_ids = check_menu(parent_ids)
-            return [x.id for x in menus
+            res = [x.id for x in menus
                     if (x.parent.id in parent_ids) or not x.parent]
 
-        res = check_menu(res)
         if count:
             return len(res)
         return res
 
     def get_action(self, ids, name):
-        action_keyword_obj = self.pool.get('ir.action.keyword')
+        pool = Pool()
+        action_keyword_obj = pool.get('ir.action.keyword')
         res = {}
         for menu_id in ids:
             res[menu_id] = False
@@ -160,7 +180,7 @@ class UIMenu(ModelSQL, ModelView):
         for action_keyword in action_keyword_obj.browse(action_keyword_ids):
             model_id = int(
                     action_keyword.model.split(',')[1].split(',')[0].strip('('))
-            action_obj = self.pool.get(action_keyword.action.type)
+            action_obj = pool.get(action_keyword.action.type)
             with Transaction().set_context(active_test=False):
                 action_id = action_obj.search([
                     ('action', '=', action_keyword.action.id),
@@ -175,7 +195,8 @@ class UIMenu(ModelSQL, ModelView):
     def set_action(self, ids, name, value):
         if not value:
             return
-        action_keyword_obj = self.pool.get('ir.action.keyword')
+        pool = Pool()
+        action_keyword_obj = pool.get('ir.action.keyword')
         action_keyword_ids = []
         cursor = Transaction().cursor
         for i in range(0, len(ids), cursor.IN_MAX):
@@ -191,7 +212,7 @@ class UIMenu(ModelSQL, ModelView):
         action_type, action_id = value.split(',')
         if not int(action_id):
             return
-        action_obj = self.pool.get(action_type)
+        action_obj = pool.get(action_type)
         action = action_obj.browse(int(action_id))
         for menu_id in ids:
             with Transaction().set_context(_timestamp=False):
