@@ -2,7 +2,6 @@
 # -*- coding: utf-8 -*-
 #This file is part of Tryton.  The COPYRIGHT file at the top level of
 #this repository contains the full copyright notices and license terms.
-from __future__ import with_statement
 import unittest
 from trytond.tests.test_tryton import POOL, DB_NAME, USER, CONTEXT, \
         install_module
@@ -23,8 +22,7 @@ class MPTTTestCase(unittest.TestCase):
             ('parent', '=', parent_id),
             ])
         childs = self.mptt.read(child_ids, ['left', 'right'])
-        childs.sort(lambda x, y: cmp(child_ids.index(x['id']),
-            child_ids.index(y['id'])))
+        childs.sort(key=lambda x: child_ids.index(x['id']))
         for child in childs:
             assert child['left'] > left, \
                     'Record (%d): left %d <= parent left %d' % \
@@ -100,7 +98,8 @@ class MPTTTestCase(unittest.TestCase):
         '''
         Create tree.
         '''
-        with Transaction().start(DB_NAME, USER, CONTEXT) as transaction:
+        with Transaction().start(DB_NAME, USER,
+                context=CONTEXT) as transaction:
             new_records = [False]
             for j in range(3):
                 parent_records = new_records
@@ -122,7 +121,8 @@ class MPTTTestCase(unittest.TestCase):
         '''
         Re-order.
         '''
-        with Transaction().start(DB_NAME, USER, CONTEXT) as transaction:
+        with Transaction().start(DB_NAME, USER,
+                context=CONTEXT) as transaction:
             self.ReOrder()
             transaction.cursor.rollback()
 
@@ -130,12 +130,14 @@ class MPTTTestCase(unittest.TestCase):
         '''
         Re-parent.
         '''
-        with Transaction().start(DB_NAME, USER, CONTEXT) as transaction:
+        with Transaction().start(DB_NAME, USER,
+                context=CONTEXT) as transaction:
             self.ReParent()
             transaction.cursor.rollback()
 
     def test0040active(self):
-        with Transaction().start(DB_NAME, USER, CONTEXT) as transaction:
+        with Transaction().start(DB_NAME, USER,
+                context=CONTEXT) as transaction:
             record_ids = self.mptt.search([])
             for record_id in record_ids:
                 if record_id % 2:
@@ -150,16 +152,18 @@ class MPTTTestCase(unittest.TestCase):
 
             transaction.cursor.rollback()
 
-        with Transaction().start(DB_NAME, USER, CONTEXT) as transaction:
+        with Transaction().start(DB_NAME, USER,
+                context=CONTEXT) as transaction:
             record_ids = self.mptt.search([])
-            self.mptt.write(record_ids[:len(record_ids)/2], {
+            self.mptt.write(record_ids[:len(record_ids) // 2], {
                     'active': False
                     })
             self.CheckTree()
 
             transaction.cursor.rollback()
 
-        with Transaction().start(DB_NAME, USER, CONTEXT) as transaction:
+        with Transaction().start(DB_NAME, USER,
+                context=CONTEXT) as transaction:
             record_ids = self.mptt.search([])
             self.mptt.write(record_ids, {
                     'active': False
@@ -175,7 +179,8 @@ class MPTTTestCase(unittest.TestCase):
         '''
         Delete.
         '''
-        with Transaction().start(DB_NAME, USER, CONTEXT) as transaction:
+        with Transaction().start(DB_NAME, USER,
+                context=CONTEXT) as transaction:
             record_ids = self.mptt.search([])
             for record_id in record_ids:
                 if record_id % 2:
@@ -184,14 +189,16 @@ class MPTTTestCase(unittest.TestCase):
 
             transaction.cursor.rollback()
 
-        with Transaction().start(DB_NAME, USER, CONTEXT) as transaction:
+        with Transaction().start(DB_NAME, USER,
+                context=CONTEXT) as transaction:
             record_ids = self.mptt.search([])
-            self.mptt.delete(record_ids[:len(record_ids)/2])
+            self.mptt.delete(record_ids[:len(record_ids) // 2])
             self.CheckTree()
 
             transaction.cursor.rollback()
 
-        with Transaction().start(DB_NAME, USER, CONTEXT) as transaction:
+        with Transaction().start(DB_NAME, USER,
+                context=CONTEXT) as transaction:
             record_ids = self.mptt.search([])
             self.mptt.delete(record_ids)
             self.CheckTree()
